"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const color_1 = require("@heroku-cli/color");
const command_1 = require("@heroku-cli/command");
const cli_ux_1 = require("cli-ux");
const util_1 = require("util");
const url_1 = require("url");
class API extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(API);
        const getBody = async () => {
            const getStdin = require('get-stdin');
            const edit = require('edit-string');
            const body = flags.body || await getStdin();
            if (!body) {
                this.warn('no stdin provided');
                return;
            }
            try {
                return JSON.parse(body);
            }
            catch (_a) {
                const err = new Error(`Request body must be valid JSON
  Received:
  ${(0, util_1.inspect)(body)}`);
                if (process.stdin.isTTY) {
                    this.warn(err);
                    return JSON.parse(await edit(body, { postfix: '.json' }));
                }
                throw err;
            }
        };
        const request = { headers: {} };
        let path = args.path;
        request.method = args.method;
        if (!path) {
            path = request.method;
            request.method = 'GET';
        }
        request.method = request.method.toUpperCase();
        const uri = this.parseURL(path);
        const version = flags.version || '3';
        request.headers.accept = `application/vnd.heroku+json; version=${version}`;
        if (flags['accept-inclusion']) {
            request.headers['Accept-Inclusion'] = flags['accept-inclusion'];
        }
        if (request.method === 'PATCH' || request.method === 'PUT' || request.method === 'POST') {
            // eslint-disable-next-line require-atomic-updates
            request.body = await getBody();
        }
        const fetch = async (body = []) => {
            cli_ux_1.default.action.start((0, color_1.default) `{cyanBright ${request.method}} ${uri.host}${uri.pathname}`);
            let response;
            try {
                response = await this.heroku.request(uri.toString(), request);
            }
            catch (error) {
                if (!error.http || !error.http.statusCode)
                    throw error;
                cli_ux_1.default.action.stop((0, color_1.default) `{redBright ${error.http.statusCode}}`);
                throw error;
            }
            let msg = (0, color_1.default) `{greenBright ${response.response.statusCode.toString()}}`;
            if (Array.isArray(response.body))
                msg += ` ${response.body.length + body.length} items`;
            cli_ux_1.default.action.stop(msg);
            if (Array.isArray(response.body) && response.response.headers['next-range']) {
                request.headers.range = response.response.headers['next-range'];
                return fetch(body.concat(response.body));
            }
            return Array.isArray(response.body) ? body.concat(response.body) : response.body;
        };
        const body = await fetch();
        if (typeof body === 'string') {
            cli_ux_1.default.log(body);
        }
        else {
            cli_ux_1.default.styledJSON(body);
        }
    }
    parseURL(path) {
        try {
            return new url_1.URL(path);
        }
        catch (error) {
            const normalizedPath = path.startsWith('/') ? path : `/${path}`;
            const protocol = this.heroku.defaults.protocol || 'https:';
            const normalizedURL = `${protocol}//${this.heroku.defaults.host}${normalizedPath}`;
            return new url_1.URL(normalizedURL);
        }
    }
}
API.description = `make a manual API request
The api command is a convenient but low-level way to send requests
to the Heroku API. It sends an HTTP request to the Heroku API
using the given method on the given path. For methods PUT, PATCH,
and POST, it uses stdin unmodified as the request body. It prints
the response unmodified on stdout.

It is essentially like curl for the Heroku API.

Method name input will be upcased, so both 'heroku api GET /apps' and
'heroku api get /apps' are valid commands.`;
API.examples = [`$ heroku api GET /apps/myapp
{
  created_at: "2011-11-11T04:17:13-00:00",
  id: "12345678-9abc-def0-1234-456789012345",
  name: "myapp",
  …
}

$ heroku api PATCH /apps/myapp/config-vars --body '{"FOO": "bar"}'
{
  FOO: "bar"
  …
}

$ export HEROKU_HEADERS
$ HEROKU_HEADERS='{
"Content-Type": "application/x-www-form-urlencoded",
"Accept": "application/json"
}'
$ printf 'type=web&qty=2' | heroku api POST /apps/myapp/ps/scale
2`];
API.flags = {
    version: command_1.flags.string({ char: 'v', description: 'version to use (e.g. 2, 3, or 3.variant)' }),
    'accept-inclusion': command_1.flags.string({ char: 'a', description: 'Accept-Inclusion header to use' }),
    body: command_1.flags.string({ char: 'b', description: 'JSON input body' }),
};
API.args = [
    { name: 'method', description: 'GET, POST, PUT, PATCH, or DELETE', required: true },
    { name: 'path', description: 'endpoint to call' },
];
exports.default = API;
